@@ -44,6 +44,8 @@ import __builtin__
 import sys
 import inspect
 import doctest
+import re
+import textwrap
 try:
     from cStringIO import StringIO
 except ImportError:
@@ -291,7 +293,7 @@ class MockTracker(AbstractTracker):
     """
     def __init__(self):
         self.mock_calls = []
-        self.mock_sets =  []
+        self.mock_sets = []
 
     def call(self, func_name, *args, **kw):
         self.mock_calls.append((func_name, args, kw))
@@ -306,19 +308,18 @@ class TraceTracker(Printer):
     closely follows the pattern of recording minimock-ed object usage as
     strings, then using the facilities of :mod:`doctest` to assert the
     correctness of these usage strings.
-    
-    
     """
     def __init__(self, *args, **kw):
         self.out = StringIO()
         super(TraceTracker, self).__init__(self.out, *args, **kw)
-        self.checker = doctest.OutputChecker()
+        self.checker = MinimockOutputChecker()
         self.options =  doctest.ELLIPSIS
-        self.options |= doctest.NORMALIZE_WHITESPACE
+        self.options |= doctest.NORMALIZE_INDENTATION
+        self.options |= doctest.NORMALIZE_FUNCTION_PARAMETERS
         self.options |= doctest.REPORT_UDIFF
         
     def check(self, want):
-        """
+        r"""
         Compare observed MiniMock usage with that which we expected.
         
         :param want: the :class:`Printer` output that results from expected
@@ -330,10 +331,12 @@ class TraceTracker(Printer):
         
             >>> tt = TraceTracker()
             >>> m = Mock('mock_obj', tracker=tt)
-            >>> m.some_meth('dummy argument')
-            >>> tt.check("Called mock_obj.some_meth('dummy argument')")
+            >>> m.some_meth('arg1')
+            >>> tt.check("Called mock_obj.some_meth('arg1')")
             True
-            >>> tt.check("Failing expected trace")
+            >>> tt.clear()
+            >>> m.some_meth('arg2')
+            >>> tt.check("does not match")
             False
         """
         return self.checker.check_output(want, self.dump(),
@@ -355,9 +358,9 @@ class TraceTracker(Printer):
             >>> tt = TraceTracker()
             >>> m = Mock('mock_obj', tracker=tt)
             >>> m.some_meth('dummy argument')
-            >>> tt.diff("Dummy string")
-            "Expected:\n    Dummy string\nGot:\n    Called mock_obj.some_meth('dummy argument')\n"
-            >>> tt.diff("Called mock_obj.some_meth('dummy argument')\n")
+            >>> tt.diff("does not match")
+            "Expected:\n    does not match\nGot:\n    Called mock_obj.some_meth('dummy argument')\n"
+            >>> tt.diff("Called mock_obj.some_meth('dummy argument')")
             ''
         """
         if self.check(want):
@@ -382,6 +385,81 @@ class TraceTracker(Printer):
         """
         return self.out.getvalue()
 
+    def clear(self):
+        """Clear the MiniMock object usage that has been tracked so far.
+        """
+        self.out.truncate(0)
+
+
+def normalize_function_parameters(text):
+    r"""
+    Return a version of ``text`` with function parameters normalized.
+
+        The normalisations performed are:
+
+        * Remove any whitespace sequence between an opening
+          parenthesis '(' and a subsequent non-whitespace character.
+
+        * Remove any whitespace sequence between a non-whitespace
+          character and a closing parenthesis ')'.
+
+        * Ensure a comma ',' and a subsequent non-whitespace character
+          are separated by a single space ' '.
+
+    Example::
+        
+        >>> tt = TraceTracker()
+        >>> foo = Mock("foo", tracker=tt)
+        >>> expect_mock_output = '''\
+        ...     Called foo.bar('baz')
+        ...     '''
+        >>> foo.bar('baz')
+        >>> tt.check(expect_mock_output)
+        True
+        >>> tt.clear()
+        >>> expect_mock_output = '''\
+        ...     Called foo.bar(
+        ...         'baz')
+        ...     '''
+        >>> foo.bar('baz')
+        >>> tt.check(expect_mock_output)
+        True
+    """
+    normalized_text = text
+    normalize_map = {
+        re.compile(r"\(\s+(\S)"): r"(\1",
+        re.compile(r"(\S)\s+\)"): r"\1)",
+        re.compile(r",\s*(\S)"): r", \1",
+        }
+    for search_pattern, replace_pattern in normalize_map.items():
+        normalized_text = re.sub(
+            search_pattern, replace_pattern, normalized_text)
+
+    return normalized_text
+
+
+doctest.NORMALIZE_INDENTATION = (
+    doctest.register_optionflag('NORMALIZE_INDENTATION'))
+doctest.NORMALIZE_FUNCTION_PARAMETERS = (
+    doctest.register_optionflag('NORMALIZE_FUNCTION_PARAMETERS'))
+
+
+class MinimockOutputChecker(doctest.OutputChecker, object):
+    """Class for matching output of MiniMock objects against expectations.
+    """
+
+    def check_output(self, want, got, optionflags):
+        if (optionflags & doctest.NORMALIZE_INDENTATION):
+            want = textwrap.dedent(want).rstrip()
+            got = textwrap.dedent(got).rstrip()
+        if (optionflags & doctest.NORMALIZE_FUNCTION_PARAMETERS):
+            want = normalize_function_parameters(want)
+            got = normalize_function_parameters(got)
+        output_match = super(MinimockOutputChecker, self).check_output(
+            want, got, optionflags)
+        return output_match
+    check_output.__doc__ = doctest.OutputChecker.check_output.__doc__
+
 
 class Mock(object):
 
